//*********************************************************************//
// this file is a translation of OSC_MIDIInput.h and OSC_MIDIInput.cpp //
//*********************************************************************//

{
	This file is a part of the OSC Library (Open Sound Code).
	Visit the OSC website at http://www.pvv.org/~thammer/HammerSound/osc/
	for the complete OSC library, the latest update, documentation,
	user forum, etc.

	Allthough the source code to the OSC Library is publicly available,
	use and distribution of the OSC Library is governed by a license agreement.
	The License Agreement can be found at the OSC website.
}


unit OSC_MIDIInput;

interface

uses
  Windows,
  mmsystem,
  OSC_Defines,
  OSC_MIDIMessage;

type
  TOSC_MIDIInput = class;
  pTOSC_MIDIInput = ^TOSC_MIDIInput;

  TOSC_MIDIInputCallbackData = record
    pseudoThis :TOSC_MIDIInput;
    dwInstance :dword;
  end;
  pTOSC_MIDIInputCallbackData = ^TOSC_MIDIInputCallbackData;

{*
	Base class for generic MIDI input. This class can be used to query the
	system about available MIDI input devices, and to open any MIDI input device
	and start receiving MIDI input messages.

	When using this class in a project, make your own MIDIInput class that inherits
	from this class, and overload the MidiInput(...) function. This function will
	be called whenever new MIDI data arrives.

	A program using this class might do something like this:
	<ul>
		<li> GetListOfDevices(...) to get the display names of all available
		MIDI input Devices</li>
		<li> Open(deviceid) - Open a device</li>
		<li> Start() - Start receiving data</li>
		<li> Process all data in the overload MidiInput(...) function</li>
		<li> Stop() - Stop receiving data</li>
		<li> Close() - Close device</li>
	</ul>

}
  TOSC_MIDIInput = class
    protected
      m_deviceid               :longint;
      m_midiInHandle           :HMIDIIN;
      m_lastMMError            :MMRESULT;
      m_midiInputCallbackData  :TOSC_MIDIInputCallbackData;
      m_open                   :boolean;
      m_start                  :boolean;
    protected
      function SetDeviceId( deviceid:longint ):OSC_ERR;
    public
      ///
      constructor Create();
      ///
      destructor Destroy();override;

      /// Result := s a list of installed MIDI input devices
      procedure GetListOfDevices( var list:StringArray; includeDeviceNumber:boolean = false );
      /// Result := s the number of installed devices
      function GetNumDevices():longint;
      /// Get the name of the current open device
      function GetDevName( var name:ansistring ):OSC_ERR;overload;
      /// Get the name of the specified device (number)
      function GetDevName(  deviceid:longint; var name:ansistring ):OSC_ERR;overload;
      /// Get the device id of the specified device (name)
      function GetDevID( var name:ansistring; var deviceid:longint ):OSC_ERR;

      function GetHandle():HMIDI;

      /// Open the device, must be done before running Start()
      function Open( deviceid:longint ):OSC_ERR;
      /// Connects MIDI In -> MIDI Out
      function Connect( midi_out_handle:HMIDIOUT ):OSC_ERR;
      /// Disconnects MIDI In -> MIDI Out
      function Disconnect( midi_out_handle:HMIDIOUT ):OSC_ERR;
      /// Close the device
      function Close():OSC_ERR;
      /// Start recieving MIDI messages. MidiInput(...) will be called for every MIDI message received by the system.
      function Start():OSC_ERR;
      /// Stop receiving MIDI messages. MidiInput(...) will no longer be called
      function Stop():OSC_ERR;

      /// Recieves the MIDI input data. Overload this function to do something useful with the MIDI Input
      procedure MidiInput( message:TOSC_MIDIMessage );virtual;

      /// Result := s the latest error (if any) from the Windows MultiMedia system
      function GetLastMMError():MMRESULT;
      /// Result := s the latest error in text format
      function GetLastMMErrorDescription():ansistring;

  end;


procedure TOSC_MIDIInputCallback( hMidiIn:HMIDIIN; wMsg:UINT; dwInstance:dword; dwParam1:dword; dwParam2:dword );stdcall;

implementation

uses SysUtils;

function TOSC_MIDIInput.Connect( midi_out_handle:HMIDIOUT ):OSC_ERR;
begin
     m_lastMMError := midiConnect( m_midiInHandle, midi_out_handle, nil );

     if ( m_lastMMError <> 0 ) then
     begin
          Result := OSC_ERR_MM;
          exit;
     end;

     Result := OSC_ERR_OK;
end;

function TOSC_MIDIInput.Disconnect( midi_out_handle:HMIDIOUT ):OSC_ERR;
begin
     m_lastMMError := midiDisconnect( m_midiInHandle, midi_out_handle, nil );

     if ( m_lastMMError <> 0 ) then
     begin
          Result := OSC_ERR_MM;
          exit;
     end;

     Result := OSC_ERR_OK;
end;

function TOSC_MIDIInput.GetLastMMError():MMRESULT;
begin
     Result := m_lastMMError;
end;

constructor TOSC_MIDIInput.Create();
begin
     inherited Create();
     m_deviceid := 0;
     m_lastMMError := 0;
     m_midiInputCallbackData.pseudoThis := self;
     m_open := false;
     m_start := false;
end;

destructor TOSC_MIDIInput.Destroy();
begin
     if ( m_start ) then Stop();
     if ( m_open ) then Close();
     inherited Destroy();
end;

function TOSC_MIDIInput.GetNumDevices():longint;
begin
     Result := midiInGetNumDevs();
end;

procedure TOSC_MIDIInput.GetListOfDevices( var list:StringArray; includeDeviceNumber:boolean);
var
   num:longint;
   caps:MIDIINCAPS;
   buf:ansistring;
   n:longint;
begin
     num := GetNumDevices();
     SetLength( list, 0 );

     n := 0;
     while ( n < num ) do
     begin
          midiInGetDevCaps( n, @caps, SizeOf( MIDIINCAPS ) );
          if ( includeDeviceNumber ) then
           buf := inttostr(n) + ' ' + caps.szPname
          else
           buf := caps.szPname;
          SetLength( list, n + 1 );
          list[n] := buf;
          inc(n);
     end;
end;

function TOSC_MIDIInput.SetDeviceId( deviceid:longint ):OSC_ERR;
begin
     if ( ( deviceid >= GetNumDevices() ) or ( deviceid < 0 ) ) then
     begin
     	  Result := OSC_ERR_ILLEGAL_DEVICE_ID;
          exit;
     end;
     m_deviceid := deviceid;
     Result := OSC_ERR_OK;
end;

function TOSC_MIDIInput.GetDevName( var name:ansistring ):OSC_ERR;
begin
     Result := GetDevName( m_deviceid, name );
end;

function TOSC_MIDIInput.GetDevName( deviceid:longint; var name:ansistring ):OSC_ERR;
var
   num:longint;
   caps:MIDIINCAPS;
begin
     num := GetNumDevices();
     if ( (deviceid < 0) or ( deviceid >= num ) ) then
     begin
     	  Result := OSC_ERR_ILLEGAL_DEVICE_ID;
          exit;
     end;
     m_lastMMError := midiInGetDevCaps( deviceid, @caps, SizeOf( MIDIOUTCAPS ) );
     if ( m_lastMMError <> 0 ) then
     begin
     	  Result := OSC_ERR_MM;
          exit;
     end;
     name := caps.szPname;
     Result := OSC_ERR_OK;
end;

function TOSC_MIDIInput.GetDevID( var name:ansistring; var deviceid:longint ):OSC_ERR;
var
   num:longint;
   ndevname:ansistring;
   found:longint;
   n:longint;
begin
     // loop through all devices and see if we can find a match
     num := GetNumDevices();
     found := -1;

     n := 0;
     while ( n < num ) do
     begin
          GetDevName( n, ndevname );
          if ( ndevname = name ) then found := n;
          inc(n);
     end;
     if ( found = -1 ) then
     begin
     	  Result := OSC_ERR_ILLEGAL_DEVICE_NAME;
          exit;
     end;

     deviceid := found;
     Result := OSC_ERR_OK;
end;

function TOSC_MIDIInput.GetHandle():HMIDI;
begin
     Result := m_midiInHandle;
end;

function TOSC_MIDIInput.Open( deviceid:longint ):OSC_ERR;
var err:OSC_ERR;
    dumb:dword;
begin
     if ( m_open ) then
     begin
          err := Close(); // close if we're allready open
          if ( err <> OSC_ERR_OK ) then
          begin
               Result := err;
               exit;
          end;
     end;

     err := SetDeviceId( deviceid );
     if ( err <> OSC_ERR_OK ) then
     begin
     	  Result := err;
          exit;
     end;

     dumb := dword(@TOSC_MIDIInputCallback);
     m_lastMMError := midiInOpen( @m_midiInHandle, m_deviceid,
      dumb, dword(@(self.m_midiInputCallbackData)), CALLBACK_FUNCTION );

     if ( m_lastMMError <> 0 ) then
     begin
     	  Result := OSC_ERR_MM;
          exit;
     end;

     m_open := true;

     Result := OSC_ERR_OK;
     exit;
end;

function TOSC_MIDIInput.Close():OSC_ERR;
begin
     if ( not m_open ) then
     begin
          Result := OSC_ERR_OK;
          exit;
     end;

     m_lastMMError := midiInClose( m_midiInHandle );

     if ( m_lastMMError <> 0 ) then
     begin
     	  Result := OSC_ERR_MM;
          exit;
     end;

     m_open := false;

     Result := OSC_ERR_OK;
end;

function TOSC_MIDIInput.Start():OSC_ERR;
begin
     if ( m_start ) then
     begin
     	  Result := OSC_ERR_OK; // We've allready been started
          exit;
     end;

     m_lastMMError := midiInStart( m_midiInHandle );

     if ( m_lastMMError <> 0 ) then
     begin
     	  Result := OSC_ERR_MM;
          exit;
     end;

     m_start := true;

     Result := OSC_ERR_OK;
end;

function TOSC_MIDIInput.Stop():OSC_ERR;
begin
     if ( not m_start ) then
     begin
          Result := OSC_ERR_OK; // We're not running
          exit;
     end;

     m_lastMMError := midiInStop( m_midiInHandle );

     if ( m_lastMMError <> 0 ) then
     begin
          Result := OSC_ERR_MM;
          exit;
     end;

     m_start := false;

     Result := OSC_ERR_OK;
end;

function TOSC_MIDIInput.GetLastMMErrorDescription():ansistring;
begin
     case m_lastMMError of
         0:                     Result := 'No Error';
         MMSYSERR_INVALHANDLE:	Result := 'The specified device handle is invalid';
         MMSYSERR_ALLOCATED:	Result := 'The specified resource is already allocated';
         MMSYSERR_BADDEVICEID:  Result := 'The specified device identifier is out of range';
         MMSYSERR_INVALFLAG:	Result := 'The flags specified by dwFlags are invalid';
         MMSYSERR_INVALPARAM:	Result := 'The specified pointer or structure is invalid';
         MMSYSERR_NOMEM:	Result := 'The system is unable to allocate or lock memory';
     else
         Result := 'Unknown Error';
     end;
end;

procedure TOSC_MIDIInput.MidiInput( message:TOSC_MIDIMessage );
begin
     // Overload this function to do something useful.
     message.Free();
end;

procedure TOSC_MIDIInputCallback( hMidiIn:HMIDIIN; wMsg:UINT; dwInstance:dword;
 dwParam1:dword; dwParam2:dword );stdcall;
var
   mididata:DWORD_4BYTES;
   data:pTOSC_MIDIInputCallbackData;
   pseudoThis:TOSC_MIDIInput;
begin
     if ( wMsg = MM_MIM_DATA ) then
     begin
     	// Convert to midi-data

     	mididata.d := dwParam1;

     	// get classpointer and call the midiInput function

     	data := pTOSC_MIDIInputCallbackData(dwInstance);
     	pseudoThis := data^.pseudoThis;

     	pseudoThis.MidiInput( TOSC_MIDIMessage.Create( mididata.b0, mididata.b1, mididata.b2,
         dwParam2 ) );
     end;
end;

end.
