//***********************************************************************//
// this file is a translation of OSC_MIDIOutput.h and OSC_MIDIOutput.cpp //
//***********************************************************************//

{
	This file is a part of the OSC Library (Open Sound Code).
	Visit the OSC website at http://www.pvv.org/~thammer/HammerSound/osc/
	for the complete OSC library, the latest update, documentation,
	user forum, etc.

	Allthough the source code to the OSC Library is publicly available,
	use and distribution of the OSC Library is governed by a license agreement.
	The License Agreement can be found at the OSC website.
}


unit OSC_MIDIOutput;

interface

uses
  Windows,
  mmsystem,
  OSC_Defines,
  OSC_MIDIMessage;

(**
	This class does MIDI output. It lets the user get a list of
	available MIDI output devices, open a MIDI output device and
	send MIDI messages to that device.

	A program using this class might do something like this:
	<ul>
		<li> GetListOfDevices(...) to get the display names of all available
		MIDI output Devices</li>
		<li> Open(deviceid) - Open a device</li>
		<li> SendMIDIMessage(...) - send cool messages</li>
		<li> Close() - Close device</li>
	</ul>

*)
type
    TOSC_MIDIOutput = class
      private
	m_deviceid:longint;
	m_midiOutHandle:HMIDIOUT;
	m_lastMMError:MMRESULT;

	m_open:boolean;

      protected
	function SetDeviceId( deviceid:longint ):OSC_ERR;
      public
	///
	constructor Create();
	///
	destructor Destroy();override;

	/// Result :=s a list of installed MIDI input devices
	procedure GetListOfDevices( var list:StringArray; includeDeviceNumber:boolean = false);
	/// Result :=s the number of installed devices
	function GetNumDevices():longint;
	/// Get the name of the current open device
	function GetDevName( var name:ansistring ):OSC_ERR;overload;
	/// Get the name of the specified device (number)
	function GetDevName( deviceid:longint; var name:ansistring ):OSC_ERR;overload;
	/// Get the device id of the specified device (name)
	function GetDevID( var name:ansistring; var deviceid:longint ):OSC_ERR;

	/// Opens the device. Must be called before calling SendMIDIMessage(...).
	function Open( deviceid:longint ):OSC_ERR;
        /// Connects MIDI In -> MIDI Out
        function Connect( midi_in_handle:HMIDI ):OSC_ERR;
        /// Disconnects MIDI In -> MIDI Out
        function Disconnect( midi_in_handle:HMIDI ):OSC_ERR;
	/// Closes the device
	function Close():OSC_ERR;

	/// Sends a MIDI message
	procedure SendMIDIMessage( status, data1, data2 :MIDIDataShort );overload;
	/// Sends a MIDI message
	procedure SendMIDIMessage( message:TOSC_MIDIMessage );overload;

	/// Result :=s the latest error (if any) from the Windows MultiMedia system
	function GetLastMMError():MMRESULT;
	/// Result :=s the latest error in text format
	function GetLastMMErrorDescription():ansistring;
        function GetHandle():HMIDIOUT;
    end;

implementation

uses SysUtils;

procedure TOSC_MIDIOutput.SendMIDIMessage( message:TOSC_MIDIMessage );
begin
     SendMIDIMessage( message.GetStatusByte(), message.GetDataByte1(),
      message.GetDataByte2() );
end;

function TOSC_MIDIOutput.GetLastMMError():MMRESULT;
begin
     Result := m_lastMMError;
end;

constructor TOSC_MIDIOutput.Create();
begin
     inherited Create();
     m_deviceid := 0;
     m_lastMMError := 0;
     m_open := false;
end;

destructor TOSC_MIDIOutput.Destroy();
begin
     if ( m_open ) then Close();
     inherited Destroy();
end;

function TOSC_MIDIOutput.Open( deviceid:longint ):OSC_ERR;
var err:OSC_ERR;
begin
     err := SetDeviceId( deviceid );
     if ( err <> OSC_ERR_OK ) then
     begin
          Result := err;
          exit;
     end;

     if ( m_open ) then
     begin
          err := Close();
          if ( err <> OSC_ERR_OK ) then
          begin
               Result := err;
               exit;
          end;
     end;

     m_lastMMError := midiOutOpen( @m_midiOutHandle, m_deviceid, 0, 0, CALLBACK_NULL);

     if ( m_lastMMError <> 0 ) then
     begin
          Result := OSC_ERR_MM;
          exit;
     end;

     m_open := true;

     Result := OSC_ERR_OK;
end;

function TOSC_MIDIOutput.Connect( midi_in_handle:HMIDI ):OSC_ERR;
begin
     m_lastMMError := midiConnect( midi_in_handle, m_midiOutHandle, nil );

     if ( m_lastMMError <> 0 ) then
     begin
          Result := OSC_ERR_MM;
          exit;
     end;

     Result := OSC_ERR_OK;
end;

function TOSC_MIDIOutput.Disconnect( midi_in_handle:HMIDI ):OSC_ERR;
begin
     m_lastMMError := midiDisconnect( midi_in_handle, m_midiOutHandle, nil );

     if ( m_lastMMError <> 0 ) then
     begin
          Result := OSC_ERR_MM;
          exit;
     end;

     Result := OSC_ERR_OK;
end;

function TOSC_MIDIOutput.Close():OSC_ERR;
begin
     if ( not m_open ) then
     begin
          Result := OSC_ERR_OK;
          exit;
     end;

     m_lastMMError := midiOutClose( m_midiOutHandle );

     if ( m_lastMMError <> 0 ) then
     begin
          Result := OSC_ERR_MM;
          exit;
     end;

     m_open := false;

     Result := OSC_ERR_OK;
end;

function TOSC_MIDIOutput.GetNumDevices():longint;
begin
     Result := midiOutGetNumDevs();
end;

procedure TOSC_MIDIOutput.GetListOfDevices( var list:StringArray; includeDeviceNumber:boolean = false );
var num:longint;
    caps:MIDIOUTCAPS;
    n:longint;
begin
     SetLength( list, 0 );
     num := GetNumDevices();

     n := 0;
     while ( n < num ) do
     begin
          midiOutGetDevCaps( n, @caps, SizeOf(MIDIOUTCAPS) );
          SetLength( list, n + 1 );
          if ( includeDeviceNumber ) then
           list[n] := inttostr(n) + ' ' + caps.szPname
          else
           list[n] := caps.szPname;
          inc(n);
     end;
end;

function TOSC_MIDIOutput.GetDevName( var name:ansistring ):OSC_ERR;
begin
     Result := GetDevName( m_deviceid, name );
end;

function TOSC_MIDIOutput.GetDevName( deviceid:longint; var name:ansistring ):OSC_ERR;
var num:longint;
    caps:MIDIOUTCAPS;
begin
     num := GetNumDevices();
     if ( ( deviceid < 0 ) or ( deviceid >= num ) ) then
     begin
          Result := OSC_ERR_ILLEGAL_DEVICE_ID;
          exit;
     end;

     m_lastMMError := midiOutGetDevCaps( deviceid, @caps, SizeOf(MIDIOUTCAPS) );
     if ( m_lastMMError <> 0 ) then
     begin
          Result := OSC_ERR_MM;
          exit;
     end;
     name := caps.szPname;
     Result := OSC_ERR_OK;
end;

function TOSC_MIDIOutput.GetHandle():HMIDIOUT;
begin
     Result := m_midiOutHandle;
end;

function TOSC_MIDIOutput.GetDevID( var name:ansistring; var deviceid:longint ):OSC_ERR;
var num:longint;
    ndevname:ansistring;
    found:longint;
    n:longint;
begin
     // loop through all devices and see if we can find a match
     num := GetNumDevices();
     found := -1;

     n := 0;
     while ( n < num ) do
     begin
          GetDevName( n, ndevname );
          if ( ndevname = name ) then found := n;
          inc(n);
     end;
     if ( found = -1 ) then
     begin
          Result := OSC_ERR_ILLEGAL_DEVICE_NAME;
          exit;
     end;

     deviceid := found;
     Result := OSC_ERR_OK;
end;


function TOSC_MIDIOutput.SetDeviceId( deviceid:longint ):OSC_ERR;
begin
     if ( deviceid >= GetNumDevices() ) then
     begin
          Result := OSC_ERR_ILLEGAL_DEVICE_ID;
          exit;
     end;
     m_deviceid := deviceid;
     Result := OSC_ERR_OK;
end;

function TOSC_MIDIOutput.GetLastMMErrorDescription():ansistring;
begin
     case m_lastMMError of
         0:		       Result := 'No Error';
         MMSYSERR_INVALHANDLE: Result := 'The specified device handle is invalid';
         MMSYSERR_ALLOCATED:   Result := 'The specified resource is already allocated';
         MMSYSERR_BADDEVICEID: Result := 'The specified device identifier is out of range';
         MMSYSERR_INVALFLAG:   Result := 'The flags specified by dwFlags are invalid';
         MMSYSERR_INVALPARAM:  Result := 'The specified pointer or structure is invalid';
         MMSYSERR_NOMEM:       Result := 'The system is unable to allocate or lock memory';
     else
         Result := 'Unknown Error';
     end;
end;

procedure TOSC_MIDIOutput.SendMIDIMessage( status, data1, data2:MIDIDataShort );
var data:DWORD_4BYTES;
begin
     // Convert 3 bytes to one double word (32 bits unsigned integer)

     data.b0 := status;
     data.b1 := data1;
     data.b2 := data2;
     data.b3 := 0;

     // send the data to the midi output device
     midiOutShortMsg( m_midiOutHandle, data.d );
end;

end.
