//***************************************************************************//
// this file is a translation of OSC_SFManager.h and OSC_SFManager.cpp files //
//***************************************************************************//

{
	This file is a part of the OSC Library (Open Sound Code).
	Visit the OSC website at http://www.pvv.org/~thammer/HammerSound/osc/
	for the complete OSC library, the latest update, documentation,
	user forum, etc.

	Allthough the source code to the OSC Library is publicly available,
	use and distribution of the OSC Library is governed by a license agreement.
	The License Agreement can be found at the OSC website.
}


unit OSC_SFManager;

interface

uses
  sfman_h,
  OSC_Defines,
  Windows;

(** This class is a wrapper class for the SoundFont
    Management System. The SoundFont Management System
    (SFMS) from Creative Labs is a device independent API
    for SoundFont compatible devices. The API lets the
    programmer load and clear SoundFonts, and retrieve
    information about loaded SoundFonts and about
    the SoundFont compatible device(s).

    To use this class, you will need to download and install
    Creative Labs' SoundFont Management System SDK
    from http://developer.soundblaster.com/ (Developer Tools).

    To fully understand all the functions in this class,
    it is recommended to read Creative Labs' description
    of the SoundFont Management System included in the above
    mentioned SDK.

    For a good example of using this class in a real world program,
    take a look at the source code for SF2LOAD.
**)

type
  TOSC_SFManager = class
    //protected
    //**** I just don't like protected class members.
    //**** They're conceptual but they're irritating. ( L.N. )
      m_hSFMANDLL                 :HINST;
      m_lpSFManagerFunctionTable  :PSFMANAGER;
      m_SFMan                     :PSFMANL101API;
      m_currentDevice             :longint;
      m_deviceOpen                :boolean;
      m_lastSFMError              :LRESULT;

      function ConnectToDLL():OSC_ERR;
      function GetSFMan():OSC_ERR;
    public
      ///
      constructor Create();
      ///
      destructor Destroy();override;
      /// Initialize the SoundFont Manager. Must be called before any of the other functions.
      function Init():OSC_ERR;
      /// Returns true if the last SFMS API call returned an error.
      function SFManError():boolean;
      /// Returns a textual description of the last SFMS error
      function GetLastSFManErrorText():ansistring;

      /// Returns the number of SoundFont compatible devices installed in the system
      function GetNumDevs():longint;
      /// Get a list of SoundFont compatible devices
      function GetListOfDevices( var list:StringArray; includeDeviceNumber:boolean = false ):OSC_ERR;
      /// Returns true if the specified device is free (and can be opened)
      function IsDeviceFree( deviceno:longint ):boolean;

      /// Open the specified device. This must be done before calling any of the functions below.
      function Open( deviceno:longint ):OSC_ERR;
      /// Close the specified device. This should be done when you're done using the device, letting other applications access the device again.
      function Close():OSC_ERR;
      /// Get the device number for the current open device
      function GetCurrentOpenDevice():longint;

      /// Has a device been opened (is this object an open device)
      function IsDeviceOpen():boolean;
      /// Load a SoundFont (from disk) into a bank (in memory accessible by the SoundFont device)
      function LoadBank( bankno:longint; path:ansistring ):OSC_ERR;
      /// Clear a loaded bank (remove SoundFont from memory)
      function ClearLoadedBank( bankno:longint ):OSC_ERR;
      /// Clear all loaded banks (frees all the memory) except the synthesizer bank (GM/GS)
      function ClearAllLoadedBanks( alsoBank0:boolean = false ):OSC_ERR;
      (** This function is used to check if a particular bank is in use.
          If the bank is in use, the function will
          Returns the number of the bank that owns the specified bank.
          If the bank is not in use, the function will return -1.
      **)
      function IsMIDIBankUsed( bankno:longint ):longint;
      (** A simplified version of IsMIDIBank used(...) that
          Returns true if the specified bank is in use
          and it is 'owned' by itself. In other words, if
          a SoundFont has been loaded specifically into this bank,
          and the bank is not owned by another bank.
      **)
      function BoolIsMIDIBankUsed( bankno:longint ):boolean;

      /// Load a preset in a SoundFont on disk into a preset in a bank
      function LoadPreset( sourceBankno, sourcePreset, destBankno, destPreset:longint; path:ansistring ):OSC_ERR;
      /// Clear a loaded preset
      function ClearLoadedPreset( bankno, preset:longint ):OSC_ERR;
      /// Load a waveform on disk into a preset in a bank
      function LoadWaveform( destBankno, destPreset:longint;
       path:ansistring ):OSC_ERR;
      /// Clear a loaded waveform
      function ClearLoadedWaveform( bankno, preset:longint ):OSC_ERR;

      /// Get the name of the SoundFont in specified bank (as specified inside the SoundFont file by the author)
      function GetLoadedBankDescriptor( bankno:longint; var desc:ansistring ):OSC_ERR;
      /// Get the path to the SoundFont in the specified bank
      function GetLoadedBankPathname( bankno:longint; var path:ansistring ):OSC_ERR;
      /// Get the name of the specified preset of the SoundFont in the specified bank
      function GetLoadedPresetDescriptor( bankno, presetno :longint; var desc:ansistring ):OSC_ERR;
      /// Get the capabilities of the specified device (you don't have to open the device first)
      function GetDevCaps( deviceno:longint; pCaps:PSFCAPSOBJECT ):OSC_ERR;overload;
      /// Get the capabilities of the current open device (see SFMS sdk doc for details)
      function GetDevCaps( pCaps:PSFCAPSOBJECT ):OSC_ERR;overload;
      /// Get the name of the speified device (you don't have to open the device first)
      function GetDevName( deviceno:longint; var name:ansistring ):OSC_ERR;overload;
      /// Get the name of the current open device
      function GetDevName( var name:ansistring ):OSC_ERR;overload;
      /// Get the device number of the specified device (name)
      function GetDevNo( name:ansistring; var deviceno:longint ):OSC_ERR;
      /// Get the maximum amount of memory and the available memory for the current open device
      function QueryStaticSampleMemorySize( var maxmem, availmem:longint ):OSC_ERR;

      /// Get a list of supported synthesizer emulations
      function GetAllSynthEmulations( var list:StringArray ):OSC_ERR;
      /// Get the name and index of the current synthesizer emulation
      function GetSynthEmulation( var index:longint; var name:ansistring ):OSC_ERR;
      /// Select the new current synthesizer emulation
      function SelectSynthEmulation( index:longint ):OSC_ERR;

      ///
      function GetNumSynthEmulations():longint;
      ///
      function GetSynthEmulationDescription( index:longint; var name:ansistring ):OSC_ERR;
end;

implementation

uses SysUtils;

constructor TOSC_SFManager.Create();
begin
     inherited Create();
     m_hSFMANDLL := 0;
     m_currentDevice := -1;
     m_deviceOpen := false;
     Init();
end;

destructor TOSC_SFManager.Destroy();
begin
     if ( IsDeviceOpen() ) then Close();
     if ( m_hSFMANDLL <> 0 ) then FreeLibrary( m_hSFMANDLL );
     inherited Destroy();
end;

function TOSC_SFManager.SFManError():boolean;
begin
     Result := ( m_lastSFMError <> SFERR_NOERR );
end;

function TOSC_SFManager.GetLastSFManErrorText():ansistring;
var ret:ansistring;
begin
     case m_lastSFMError of
       SFERR_NOERR                 :     ret := '';
       SFERR_UNDEFINE_ERROR        :     ret := 'Undefined Error';
       SFERR_LOAD_DLL_FAIL         :     ret := 'Load Dll Failed';
       SFERR_GET_ENTRYPOINT_FAIL   :     ret := 'Get Entrypoint Fail';
       SFERR_DEVICE_NOT_FOUND      :     ret := 'Device Not Found';
       SFERR_INTERFACE_NOT_SUPPORTED:    ret := 'Interface Not Supported';
       SFERR_LAST_API              :     ret := 'Last API';
       SFERR_SYNTH_INVALID         :     ret := 'Synth Invalid';
       SFERR_BANK_INDEX_INVALID    :     ret := 'Bank Index Invalid';
       SFERR_PATHNAME_INVALID      :     ret := 'Pathname Invalid';
       SFERR_FORMAT_INVALID        :     ret := 'Format Invalid';
       SFERR_PRESET_INDEX_INVALID  :     ret := 'Preset Index Invalid';
       SFERR_ACCESS_NOT_PERMITTED  :     ret := 'Access Not Permitted';
       SFERR_VERSION_INVALID       :     ret := 'Version Invalid';
       SFERR_DEVICE_INVALID        :     ret := 'Device Invalid';
       SFERR_SUPPORT_INVALID       :     ret := 'Support Invalid';
       SFERR_INVALID_SOUND_ROM     :     ret := 'Invalid Sound Rom';
       SFERR_NOT_STATIC_MEMORY     :     ret := 'Not Static Memory';
       SFERR_LAST_INVALID          :     ret := 'Last Invalid';
       SFERR_DLL_BUSY              :     ret := 'Dll Busy';
       SFERR_DEVICE_BUSY           :     ret := 'Device Busy';
       SFERR_RESOURCE_INSUFFICIENT :     ret := 'Resource Insufficient';
       SFERR_SYSMEM_INSUFFICIENT   :     ret := 'System Memory Insufficient';
       SFERR_SOUNDMEM_INSUFFICIENT :     ret := 'Sound Memory Insufficient';
       SFERR_LAST_CONTENTION       :     ret := 'Last Contention';
     else
       ret := 'Unknown Error';
     end;
     Result := ret;
end;

function TOSC_SFManager.ConnectToDLL():OSC_ERR;
begin
     if ( m_hSFMANDLL <> 0 ) then FreeLibrary( m_hSFMANDLL );
     m_hSFMANDLL := 0;
     m_hSFMANDLL := LoadLibrary( SF_MASTER_MANAGER_FILENAME );
     if ( m_hSFMANDLL = 0 ) then
      Result := OSC_ERR_FAIL
     else
      Result := OSC_ERR_OK;
end;

function TOSC_SFManager.GetSFMan():OSC_ERR;
var res:OSC_ERR;
begin
     res := ConnectToDLL();
     if ( res <> OSC_ERR_OK ) then 
     begin
          Result := res;
          exit;
     end;

     // Get the address of the SoundFont manager function table
     m_lpSFManagerFunctionTable :=
      PSFManager(GetProcAddress( m_hSFMANDLL, SF_FUNCTION_TABLE_NAME ));

     // If this fails, unload the library and report an error
     if ( m_lpSFManagerFunctionTable = nil ) then
     begin
          Result := OSC_ERR_SFMAN_COULDNT_GET_SFMAN_FUNCTION_TABLE;
          exit;
     end;

     // Use SF_QueryInterface to get access to the appropriate function table
     if ( m_lpSFManagerFunctionTable^.SF_QueryInterface(
      ID_SFMANL101API, PDWORD(@m_SFMan) ) <> 0 ) then
     begin
          Result := OSC_ERR_SFMAN_COULDNT_GET_SFMAN_INTERFACE;
          exit;
     end;

     Result := OSC_ERR_OK;
end;

function TOSC_SFManager.Init():OSC_ERR;
var res:OSC_ERR;
begin
     res := GetSFMan();
     Result := res;
end;

function TOSC_SFManager.GetNumDevs():longint;
var numSFDevs:word;
begin
     m_lastSFMError := m_SFMan^.SF_GetNumDevs( @numSFDevs );
     Result := numSFDevs;
end;

function TOSC_SFManager.GetListOfDevices( var list:StringArray;
 includeDeviceNumber:boolean = false ):OSC_ERR;
var res:OSC_ERR;
    caps:CSFCapsObject;
    i:longint;
begin
     res := OSC_ERR_OK;
     SetLength( list, 0 );
     i := 0;
     while ( i < GetNumDevs() ) do
     begin
          m_lastSFMError := GetDevCaps( i, @caps );
          if ( m_lastSFMError <> SFERR_NOERR ) then
          begin
               res := OSC_ERR_FAIL;
               break;
          end;
          SetLength( list, i + 1 );
          if ( includeDeviceNumber ) then
           list[i] := inttostr( i ) + ' ' + caps.m_DevName
          else
           list[i] := caps.m_DevName;
          inc(i);
     end;
     Result := res;
end;

function TOSC_SFManager.Open( deviceno:longint ):OSC_ERR;
begin
     if ( ( deviceno < 0 ) or ( deviceno >= GetNumDevs() ) ) then
     begin
          Result := OSC_ERR_SFMAN_INVALID_DEVICE_NUMBER;
          exit;
     end;
     if ( IsDeviceOpen() ) then Close();
     m_currentDevice := deviceno;
     m_deviceOpen := true;
     m_lastSFMError := m_SFMan^.SF_Open( m_currentDevice );
     if ( m_lastSFMError = SFERR_NOERR ) then
     begin
          Result := OSC_ERR_OK;
     end
     else
     begin
          Result := OSC_ERR_FAIL;
     end;
end;

function TOSC_SFManager.Close():OSC_ERR;
begin
     m_lastSFMError := m_SFMan^.SF_Close( m_currentDevice );
     m_currentDevice := -1;
     m_deviceOpen := false;
     if ( m_lastSFMError = SFERR_NOERR ) then
     begin
          Result := OSC_ERR_OK;
     end
     else
     begin
          Result := OSC_ERR_FAIL;
     end;
end;

function TOSC_SFManager.IsDeviceOpen():boolean;
begin
     Result := m_deviceOpen;
end;

function TOSC_SFManager.GetCurrentOpenDevice():longint;
begin
     Result := m_currentDevice;
end;

function TOSC_SFManager.IsDeviceFree( deviceno:longint ):boolean;
begin
     m_lastSFMError := m_SFMan^.SF_IsDeviceFree( deviceno );
     Result := ( m_lastSFMError = SFERR_NOERR );
end;

function TOSC_SFManager.LoadBank( bankno:longint; path:ansistring ):OSC_ERR;
var midi:CSFMIDILocation;
    buf:CSFBufferObject;
    textbuf:array[ 0.. 1023 ] of char;
begin
     textbuf[ 1023 ] := #0;
     StrLCopy( pChar(@textbuf[0]), pChar(path), 1023 );
     midi.m_BankIndex := bankno;
     buf.m_Flag := ord(SFFLAG_OPER_FILE);
     buf.m_Buffer := @(textbuf[0]);
     buf.m_Size := StrLen( pChar(@textbuf[0]) ) + 1;
     m_lastSFMError := m_SFMan^.SF_LoadBank( m_currentDevice, @midi, @buf );
     if ( m_lastSFMError = SFERR_NOERR ) then
     begin
          Result := OSC_ERR_OK;
     end
     else
     begin
          Result := OSC_ERR_FAIL;
     end;
end;

function TOSC_SFManager.ClearLoadedBank( bankno:longint ):OSC_ERR;
var midi:CSFMIDILocation;
    done:boolean;
begin
     midi.m_BankIndex := bankno;
     done := false;
     while ( not done ) do
     begin
          m_lastSFMError := m_SFMan^.SF_ClearLoadedBank( m_currentDevice, @midi );
          if ( m_lastSFMError <> OSC_ERR_OK ) then
          begin
               Result := OSC_ERR_FAIL;
               exit;
          end;
          if ( IsMIDIBankUsed( bankno ) <> bankno ) then done := true;
     end;
     Result := OSC_ERR_OK;
end;

function TOSC_SFManager.ClearAllLoadedBanks( alsoBank0:boolean = false ):OSC_ERR;
var res:OSC_ERR;
    start:longint;
    i:longint;
begin
     // Note: Will not clear bank 0, the emulation bank
     res := OSC_ERR_OK;
     if ( alsoBank0 ) then start := 0 else start := 1;

     i := start;
     while ( i <= 127 ) do
     begin
          if ( IsMIDIBankUsed( i ) = i ) then
          begin
               res := ClearLoadedBank( i );
               if ( res <> OSC_ERR_OK ) then break;
          end;
     end;
     Result := res;
end;

function TOSC_SFManager.IsMIDIBankUsed( bankno:longint ):longint;
var index:word;
begin
     index := bankno;
     m_lastSFMError := m_SFMan^.SF_IsMIDIBankUsed( m_currentDevice, @index );
     if ( index = 65535 ) then Result := -1 else Result := index;
end;

function TOSC_SFManager.BoolIsMIDIBankUsed( bankno:longint ):boolean;
begin
     Result := ( IsMIDIBankUsed( bankno ) = bankno );
end;


function TOSC_SFManager.GetLoadedBankDescriptor( bankno:longint; var desc:ansistring ):OSC_ERR;
var midi:CSFMIDILocation;
    buf:CSFBufferObject;
    textbuf:array[ 0.. 1023 ] of char;
begin
     textbuf[ 1023 ] := #0;
     midi.m_BankIndex := bankno;
     buf.m_Buffer := @(textbuf[0]);
     buf.m_Size := 1023;

     m_lastSFMError := m_SFMan^.SF_GetLoadedBankDescriptor( m_currentDevice, @midi, @buf );

     if ( m_lastSFMError = SFERR_NOERR ) then desc := buf.m_Buffer else desc := '';

     if ( m_lastSFMError = SFERR_NOERR ) then
     begin
          Result := OSC_ERR_OK;
     end
     else
     begin
          Result := OSC_ERR_FAIL;
     end;
end;

function TOSC_SFManager.GetLoadedBankPathname( bankno:longint; var path:ansistring ):OSC_ERR;
var midi:CSFMIDILocation;
    buf:CSFBufferObject;
    textbuf:array[ 0.. 1023 ] of char;
begin
     textbuf[ 1023 ] := #0;
     midi.m_BankIndex := bankno;
     buf.m_Buffer := @(textbuf[0]);
     buf.m_Size := 1023;

     m_lastSFMError := m_SFMan^.SF_GetLoadedBankPathname( m_currentDevice, @midi, @buf );

     if ( m_lastSFMError = SFERR_NOERR ) then path := buf.m_Buffer else path := '';

     if ( m_lastSFMError = SFERR_NOERR ) then
     begin
          Result := OSC_ERR_OK;
     end
     else
     begin
          Result := OSC_ERR_FAIL;
     end;
end;

function TOSC_SFManager.GetLoadedPresetDescriptor( bankno, presetno :longint;
 var desc:ansistring ):OSC_ERR;
var midi:CSFMIDILocation;
    buf:CSFBufferObject;
    textbuf:array[ 0.. 1023 ] of char;
begin
     textbuf[ 1023 ] := #0;
     midi.m_BankIndex := bankno;
     buf.m_Buffer := @(textbuf[0]);
     buf.m_Size := 1023;
     midi.m_PresetIndex := presetno;

     m_lastSFMError := m_SFMan^.SF_GetLoadedPresetDescriptor( m_currentDevice, @midi, @buf);

     if ( m_lastSFMError = SFERR_NOERR ) then
     begin
          if ( ( buf.m_SizeUsed > 1 ) or ( textbuf[0] <> #0 ) ) then
           desc := buf.m_Buffer
          else
           desc := '';
     end
     else
     begin
          desc := '';
     end;

     if ( m_lastSFMError = SFERR_NOERR ) then
     begin
          Result := OSC_ERR_OK;
     end
     else
     begin
          Result := OSC_ERR_FAIL;
     end;
end;

function TOSC_SFManager.GetDevCaps( pCaps:PSFCAPSOBJECT ):OSC_ERR;
begin
     m_lastSFMError := m_SFMan^.SF_GetDevCaps( m_currentDevice, pCaps );

     if (m_lastSFMError = SFERR_NOERR) then
          Result := OSC_ERR_OK
     else
          Result := OSC_ERR_FAIL;
end;

function TOSC_SFManager.GetDevCaps( deviceno:longint; pCaps:PSFCAPSOBJECT ):OSC_ERR;
begin
     m_lastSFMError := m_SFMan^.SF_GetDevCaps(deviceno, pCaps);

     if ( m_lastSFMError = SFERR_NOERR ) then
     begin
          Result := OSC_ERR_OK;
     end
     else
     begin
          Result := OSC_ERR_FAIL;
     end;
end;

function TOSC_SFManager.GetDevName( var name:ansistring ):OSC_ERR;
begin
     Result := GetDevName( m_currentDevice, name );
end;

function TOSC_SFManager.GetDevName( deviceno:longint; var name:ansistring ):OSC_ERR;
var res:OSC_ERR;
    caps:CSFCapsObject;
begin
     res := GetDevCaps( deviceno, @caps );
     if ( res <> OSC_ERR_OK ) then
     begin
          Result := res;
          exit;
     end;
     name := caps.m_DevName;
     Result := res;
end;

function TOSC_SFManager.GetDevNo( name:ansistring; var deviceno:longint ):OSC_ERR;
var num:longint;
    ndevname:ansistring;
    found:longint;
    n:longint;
begin
     // loop through all devices and see if we can find a match
     num := GetNumDevs();
     found := -1;

     n := 0;
     while ( n < num ) do
     begin
          GetDevName( n, ndevname );
          if ( ndevname = name ) then found := n;
          inc(n);
     end;
     if ( found = -1 ) then 
     begin
          Result := OSC_ERR_ILLEGAL_DEVICE_NAME;
          exit;
     end;

     deviceno := found;
     Result := OSC_ERR_OK;
end;

function TOSC_SFManager.QueryStaticSampleMemorySize( var maxmem, availmem :longint ):OSC_ERR;
var max,avail:dword;
begin
     m_lastSFMError :=
      m_SFMan^.SF_QueryStaticSampleMemorySize( m_currentDevice, @max, @avail);
     maxmem := max;
     availmem := avail;

     if ( m_lastSFMError = SFERR_NOERR ) then
     begin
          Result := OSC_ERR_OK;
     end
     else
     begin
          Result := OSC_ERR_FAIL;
     end;
end;


function TOSC_SFManager.LoadPreset( sourceBankno, sourcePreset, destBankno, destPreset:longint;
 path:ansistring ):OSC_ERR;
var midisource:CSFMIDILocation;
    mididest:CSFMIDILocation;
    buf:CSFBufferObject;
    textbuf:array[ 0.. 1023 ] of char;
begin
     textbuf[ 1023 ] := #0;
     midisource.m_BankIndex := sourceBankno;
     midisource.m_PresetIndex := sourcePreset;
     mididest.m_BankIndex := destBankno;
     mididest.m_PresetIndex := destPreset;
     StrLCopy( pChar(@textbuf[0]), pChar(path), 1023 );
     buf.m_Buffer := @(textbuf[0]);
     buf.m_Size := StrLen( pChar(@(textbuf[0])) ) + 1;
     buf.m_Flag := ord(SFFLAG_OPER_FILE);
     m_lastSFMError :=
      m_SFMan^.SF_LoadPreset( m_currentDevice, @midisource, @mididest, @buf );
     if ( m_lastSFMError = SFERR_NOERR ) then
     begin
          Result := OSC_ERR_OK;
     end
     else
     begin
          Result := OSC_ERR_FAIL;
     end;
end;

function TOSC_SFManager.ClearLoadedPreset( bankno, preset :longint ):OSC_ERR;
var midi:CSFMIDILocation;
begin
     midi.m_BankIndex := bankno;
     midi.m_PresetIndex := preset;
     m_lastSFMError := m_SFMan^.SF_ClearLoadedPreset( m_currentDevice, @midi );
     if ( m_lastSFMError <> OSC_ERR_OK ) then
     begin
          Result := OSC_ERR_FAIL;
          exit;
     end;
     Result := OSC_ERR_OK;
end;

function TOSC_SFManager.LoadWaveform( destBankno, destPreset :longint;
 path:ansistring ):OSC_ERR;
var mididest:CSFMIDILocation;
    buf:CSFBufferObject;
    textbuf:array[ 0.. 1023 ] of char;
begin
     textbuf[ 1023 ] := #0;
     StrLCopy( pChar(@textbuf[0]), pChar(path), 1023 );
     mididest.m_BankIndex := destBankno;
     mididest.m_PresetIndex := destPreset;
     buf.m_Flag := ord(SFFLAG_OPER_FILE);
     buf.m_Buffer := @(textbuf[0]);
     buf.m_Size := StrLen( pChar(@(textbuf[0])) ) + 1;
     m_lastSFMError := m_SFMan^.SF_LoadWaveform( m_currentDevice, @mididest, @buf );
     if ( m_lastSFMError = SFERR_NOERR ) then
     begin
          Result := OSC_ERR_OK;
     end
     else
     begin
          Result := OSC_ERR_FAIL;
     end;
end;

function TOSC_SFManager.ClearLoadedWaveform( bankno, preset:longint ):OSC_ERR;
var midi:CSFMIDILocation;
begin
     midi.m_BankIndex := bankno;
     midi.m_PresetIndex := preset;
     m_lastSFMError := m_SFMan^.SF_ClearLoadedWaveform( m_currentDevice, @midi );
     if ( m_lastSFMError <> OSC_ERR_OK ) then
     begin
          Result := OSC_ERR_FAIL;
          exit;
     end;
     Result := OSC_ERR_OK;
end;

function TOSC_SFManager.GetAllSynthEmulations( var list:StringArray ):OSC_ERR;
var buf:CSFBufferObject;
    textbuf:array[ 0.. 2047 ] of char;
    current:ansistring;
    char2str:array[ 0.. 1 ] of char;
    i:longint;
begin
     textbuf[ 2047 ] := #0;
     buf.m_Buffer := @(textbuf[0]);
     buf.m_Size := 2047;

     m_lastSFMError := m_SFMan^.SF_GetAllSynthEmulations( m_currentDevice, @buf );

     if ( m_lastSFMError = SFERR_NOERR ) then
     begin
          SetLength( list, 0 );
          current := '';
          char2str[ 1 ] := #0;
          i := 0;
          while ( dword(i) < ( buf.m_SizeUsed - 1 ) ) do
          begin
               if ( buf.m_Buffer[ i ] <> #0 ) then
               begin
                    char2str[ 0 ] := buf.m_Buffer[ i ];
                    current := current + char2str;
               end
               else
               begin
                    SetLength( list, Length( list ) + 1 );
                    list[ Length( list ) - 1 ] := current;
                    current := '';
               end;
               inc(i);
          end;
          Result := OSC_ERR_OK;
          exit;
     end
     else
     begin
          Result := OSC_ERR_FAIL;
          exit;
     end;
     Result := OSC_ERR_FAIL;
end;

function TOSC_SFManager.GetSynthEmulation( var index:longint; var name:ansistring ):OSC_ERR;
var buf:CSFBufferObject;
    textbuf:array[ 0.. 1023 ] of char;
begin
     textbuf[ 1023 ] := #0;
     buf.m_Buffer := @(textbuf[0]);
     buf.m_Size := 1023;

     m_lastSFMError := m_SFMan^.SF_GetSynthEmulation( m_currentDevice, @buf );

     if ( m_lastSFMError = SFERR_NOERR ) then
     begin
          index := buf.m_Flag;
          name := buf.m_Buffer;
          Result := OSC_ERR_OK;
          exit;
     end
     else
     begin
          Result := OSC_ERR_FAIL;
          exit;
     end;
end;

function TOSC_SFManager.SelectSynthEmulation( index:longint ):OSC_ERR;
begin
     m_lastSFMError := m_SFMan^.SF_SelectSynthEmulation( m_currentDevice, index );
     if ( m_lastSFMError <> OSC_ERR_OK ) then
     begin
          Result := OSC_ERR_FAIL;
          exit;
     end;
     Result := OSC_ERR_OK;
end;

function TOSC_SFManager.GetNumSynthEmulations():longint;
var list:StringArray;
    err:OSC_ERR;
begin
     err := GetAllSynthEmulations( list );
     if ( err <> OSC_ERR_OK ) then
      Result := 0
     else
      Result := Length( list );
end;

function TOSC_SFManager.GetSynthEmulationDescription( index:longint;
 var name:ansistring ):OSC_ERR;
var list:StringArray;
    err:OSC_ERR;
begin
     err := GetAllSynthEmulations( list );
     if ( err <> OSC_ERR_OK ) then
     begin
          Result := err;
          exit;
     end;
     if ( ( index < 0 ) or ( index >= Length( list ) ) ) then
     begin
          Result := OSC_ERR_FAIL; // wrong index
          exit;
     end;
     name := list[ index ];
     Result := OSC_ERR_OK;
end;

end.
